function [Fitness,D] = CalFitnessDecEpsilon(PopObj,PopDec,epsilon)
% Calculate the fitness of each solution based on epsilon objective space

    N = size(PopObj,1);

    %% Detect the dominance relation between each two solutions
    Dominate = false(N);
    for i = 1 : N-1
        for j = i+1 : N
            k = any(PopObj(j,:)-PopObj(i,:) > epsilon) - any(PopObj(j,:)-PopObj(i,:) < epsilon);
            if k == 1
                Dominate(i,j) = true;
            elseif k == -1
                Dominate(j,i) = true;
            end
        end
    end
    
    %% Calculate S(i)
    S = sum(Dominate,2);
    
    %% Calculate R(i)
    R = zeros(1,N);
    for i = 1 : N
        R(i) = sum(S(Dominate(:,i)));
    end
    
    %% Calculate D(i)
    Distance = pdist2(PopDec,PopDec);
    Distance(logical(eye(length(Distance)))) = inf;
    Distance = sort(Distance,2);
    D_Dec = 1./(Distance(:,floor(sqrt(N)))+2);
    
    D = D_Dec;
    
    %% Calculate the fitnesses
    Fitness = R + D';
end